@@ -1,15 +1,14 @@
-;;; es-mode.el --- A major mode for editing Elasticsearch queries
-;; -*- coding: utf-8; lexical-binding: t; -*-
+;;; es-mode.el --- A major mode for editing and executing Elasticsearch queries -*- coding: utf-8; lexical-binding: t; -*-
 
 ;; Copyright (C) 2014 Matthew Lee Hinman
 ;; Copyright (C) 2014 Bjarte Johansen
 
 ;; Author: Lee Hinman <lee@writequit.org>
 ;; URL: http://www.github.com/dakrone/es-mode
-;; Package-Version: 4.3.0
-;; Package-Revision: 4.3.0-0-g996730ebce57
+;; Package-Version: 20260220.2147
+;; Package-Revision: cf93c9900057
 ;; Keywords: elasticsearch
-;; Package-Requires: ((dash "2.11.0") (cl-lib "0.5") (spark "1.0"))
+;; Package-Requires: ((dash "2.11.0") (cl-lib "0.5") (spark "1.0") (s "1.11.0") (request "0.3.0"))
 
 ;; This file is not part of GNU Emacs.
 
@@ -51,6 +50,8 @@
 (require 'url-handlers)
 (require 'url-parse)
 (require 'url-util)
+(require 'request)
+(require 'subr-x)
 
 (defgroup es nil
   "Major mode for editing Elasticsearch queries."
@@ -66,6 +67,15 @@
   :group 'es
   :type 'string)
 
+(defcustom es-default-headers
+  '(("Content-Type" . "application/json; charset=UTF-8"))
+  "The default request headers. Note that removing the
+`Content-Type' header from this list may cause Elasticsearch to
+throw exceptions, it is recommended that you only *add* to this
+list"
+  :group 'es
+  :type 'list)
+
 (defcustom es-prompt-url nil
   "Non-nil means prompt user for requested URL on each query
   evaluation."
@@ -141,13 +151,15 @@ the user on DELETE requests."
 
 (eval-and-compile
   (defvar es-top-level-fields
-    '("aggregations" "aggs" "facets" "filter"
-      "post_filter" "query")
+    '("aggregations" "aggs" "facets" "filter" "post_filter" "query" "highlight"
+      "rescore" "scroll" "scroll_id" "indices_boost" "min_score")
     "Top-level query and filter containers"))
 
 (eval-and-compile
   (defvar es-keywords
-    '("fields" "from" "size" "highlight" "_name" "_cache" "_cache_key")
+    '("fields" "from" "size" "highlight" "_name" "_cache" "_cache_key"
+      "_source" "script_fields" "docvalue_fields" "explain" "version" "collapse"
+      "search_after" "profile")
     "Top-level fields supported by all queries")
 
   (defvar es-warnings
@@ -174,6 +186,8 @@ the user on DELETE requests."
         (:type "parent" :summary "Parent combining multiple filters/queries, prefer <bool>"))
       #("not" 0 1
         (:type "parent" :summary "Parent combining multiple filters/queries, prefer <bool>"))
+      #("constant_score" 0 1
+        (:type "parent" :summary "Parent query wrapping a filter returning a constant score value"))
 
 ;;; Both queries and filter
       #("term" 0 1
@@ -200,8 +214,6 @@ the user on DELETE requests."
         (:type "query" :summary "Query promoting or demoting results matching a query"))
       #("common" 0 1
         (:type "query" :summary "Query with cutoff for common terms"))
-      #("constant_score" 0 1
-        (:type "query" :summary "Query wrapping a filter returning a constant score value"))
       #("dis_max" 0 1
         (:type "query" :summary "Query for disjuntive max of multiple queries"))
       #("fuzzy_like_this" 0 1
@@ -234,6 +246,8 @@ the user on DELETE requests."
         (:type "query" :summary "Matches spans containing a term"))
       #("wildcard" 0 1
         (:type "query" :summary "Query matching documents that have fields matching a wildcard expression (not analyzed)"))
+      #("inner_hits" 0 1
+        (:type "query" :summary "Return the matched child or nested documents"))
       #("top_children" 0 1
         (:type "filter" :summary "Execute a child query, and out of the hit docs, aggregates it into parent docs"))
 
@@ -272,55 +286,71 @@ the user on DELETE requests."
       #("type" 0 1
         (:type "filter" :summary "Filter based on document type"))
 
-;;; Aggregations
+;;; Metric Aggregations
       #("min" 0 1
-        (:type "agg" :summary "Aggregation for minimum value"))
+        (:type "metric agg" :summary "Aggregation for minimum value"))
       #("max" 0 1
-        (:type "agg" :summary "Aggregation for maximum value"))
+        (:type "metric agg" :summary "Aggregation for maximum value"))
       #("sum" 0 1
-        (:type "agg" :summary "Aggregation for sum of values"))
+        (:type "metric agg" :summary "Aggregation for sum of values"))
       #("avg" 0 1
-        (:type "agg" :summary "Aggregation for average of values"))
+        (:type "metric agg" :summary "Aggregation for average of values"))
       #("stats" 0 1
-        (:type "agg" :summary "Aggregation calculating statistics of numeric values"))
+        (:type "metric agg" :summary "Aggregation calculating statistics of numeric values"))
       #("extended_stats" 0 1
-        (:type "agg" :summary "Aggregation calculating extended statistics of numeric values"))
+        (:type "metric agg" :summary "Aggregation calculating extended statistics of numeric values"))
       #("value_count" 0 1
-        (:type "agg" :summary "Aggregation counting number of values extracted from field"))
+        (:type "metric agg" :summary "Aggregation counting number of values extracted from field"))
       #("percentiles" 0 1
-        (:type "agg" :summary "Aggregation calculating percentiles of numeric values"))
+        (:type "metric agg" :summary "Aggregation calculating percentiles of numeric values"))
       #("percentile_ranks" 0 1
-        (:type "agg" :summary "Aggregation calculating percentile rank of numeric values"))
+        (:type "metric agg" :summary "Aggregation calculating percentile rank of numeric values"))
       #("cardinality" 0 1
-        (:type "agg" :summary "Aggregation calculating cardinality of a field"))
+        (:type "metric agg" :summary "Aggregation calculating cardinality of a field"))
       #("geo_bounds" 0 1
-        (:type "agg" :summary "Aggregation within geo bounding box"))
+        (:type "metric agg" :summary "Aggregation within geo bounding box"))
       #("top_hits" 0 1
-        (:type "agg" :summary "Aggregation of results within a bucket (join)"))
-      #("global" 0 1
-        (:type "agg" :summary "Aggregation returning all results regardless of scope"))
-      #("reverse_nested" 0 1
-        (:type "agg" :summary "Aggregation for reverse nested documents"))
-      #("terms" 0 1
-        (:type "agg" :summary "Aggregation calculating most or least common terms"))
+        (:type "metric agg" :summary "Aggregation of results within a bucket (join)"))
       #("significant_terms" 0 1
-        (:type "agg" :summary "Aggregation returning interesting or unusual occurrences of terms in a set"))
-      #("range" 0 1
-        (:type "agg" :summary "Aggregation of documents within ranges"))
-      #("date_range" 0 1
-        (:type "agg" :summary "Aggregation of documents within a date range"))
-      #("ip_range" 0 1
-        (:type "agg" :summary "Aggregation of documents within an IP address range"))
-      #("missing" 0 1
-        (:type "agg" :summary "Aggregation of documents missing a field value"))
-      #("histogram" 0 1
-        (:type "agg" :summary "Aggregation of documents within numeric slices"))
-      #("date_histogram" 0 1
-        (:type "agg" :summary "Aggregation of documents within date slices"))
-      #("filters" 0 1
-        (:type "agg" :summary "Aggregation bucketing documents into buckets defined by filters"))
+        (:type "metric agg" :summary "Aggregation returning interesting or unusual occurrences of terms in a set"))
 
-;;; Pipeline aggregations
+;;; Bucket Aggregations
+      #("global" 0 1
+        (:type "bucket agg" :summary "Aggregation returning all results regardless of scope"))
+      #("terms" 0 1
+        (:type "bucket agg" :summary "Aggregation calculating most or least common terms"))
+      #("reverse_nested" 0 1
+        (:type "bucket agg" :summary "Aggregation for reverse nested documents"))
+      #("range" 0 1
+        (:type "bucket agg" :summary "Aggregation of documents within ranges"))
+      #("date_range" 0 1
+        (:type "bucket agg" :summary "Aggregation of documents within a date range"))
+      #("ip_range" 0 1
+        (:type "bucket agg" :summary "Aggregation of documents within an IP address range"))
+      #("geohash_grid" 0 1
+        (:type "bucket agg" :summary "Aggregation that works on geo_point fields and groups points into buckets that represent cells in a grid"))
+      #("missing" 0 1
+        (:type "bucket agg" :summary "Aggregation of documents missing a field value"))
+      #("histogram" 0 1
+        (:type "bucket agg" :summary "Aggregation of documents within numeric slices"))
+      #("date_histogram" 0 1
+        (:type "bucket agg" :summary "Aggregation of documents within date slices"))
+      #("filters" 0 1
+        (:type "bucket agg" :summary "Aggregation bucketing documents into buckets defined by filters"))
+      #("sampler" 0 1
+        (:type "bucket agg" :summary "Aggregation used to limit any sub aggregations' processing to a sample of the top-scoring documents"))
+      #("diversified_sampler" 0 1
+        (:type "bucket agg" :summary "Aggregation used to limit any sub aggregations' processing to a sample of the top-scoring documents"))
+      #("children" 0 1
+        (:type "bucket agg" :summary "Aggregation that enables aggregating from buckets on parent document types to buckets on child documents"))
+
+;;; Matrix Aggregations
+      #("adjacency_matrix" 0 1
+        (:type "matrix agg" :summary "A matrix aggregation returning a form of adjacency matrix"))
+      #("matrix_stats" 0 1
+        (:type "matrix agg" :summary "A matrix aggregation calculating numeric aggregation over a set of document fields"))
+
+;;; Pipeline Aggregations
       #("avg_bucket" 0 1
         (:type "pipeline agg" :summary "Calculates the (mean) average value of a specified metric in a sibling aggregation"))
       #("derivative" 0 1
@@ -347,6 +377,14 @@ the user on DELETE requests."
         (:type "pipeline agg" :summary "Executes a script which determines whether the current bucket will be retained in the parent multi-bucket aggregation"))
       #("serial_diff" 0 1
         (:type "pipeline agg" :summary "A technique where values in a time series are subtracted from itself at different time lags or periods"))
+
+;;; Bool
+      #("must" 0 1
+        (:type "bool" :summary "The clause (query) must appear in matching documents and will contribute to the score."))
+      #("should" 0 1
+        (:type "bool" :summary "The clause (query) can optionaly appear in the matching document and will contribute to the score."))
+      #("must_not" 0 1
+        (:type "bool" :summary "The clause (query) must not appear in the matching documents."))
       )
     "Vars used for query and filter completion"))
 
@@ -382,7 +420,7 @@ the user on DELETE requests."
 (defconst es--method-url-regexp
   (concat "^\\("
           (regexp-opt es-http-builtins-all)
-          "\\) \\(.*\\)$")
+          "\\)[[:space:]]+\\(.*\\)$")
   "A regex to get the method and url from a line.")
 
 (defun es-set-endpoint-url (new-url)
@@ -434,7 +472,8 @@ in which case it prompts the user."
 
 (defun es--fix-url (url)
   "Transforms the URL so that we can use it to send a request."
-  (cond ((not (string-prefix-p "http://" url))
+  (cond ((and (not (string-prefix-p "http://" url t))
+              (not (string-prefix-p "https://" url t)))
          (let ((url (if (string-prefix-p "/" url)
                         url
                       (concat "/" url))))
@@ -456,12 +495,12 @@ in which case it prompts the user."
 (defun es--find-params ()
   "Search backwards to find text like \"POST /_search\",
   returning a list of method and full URL, prepending
-  `es-default-base' to the URL. Returns `false' if no parameters
+  `es-default-url' to the URL. Returns `false' if no parameters
   are found."
   (save-excursion
     (if (search-backward-regexp es--method-url-regexp nil t)
-        (let ((method (match-string 1))
-              (uri (match-string 2)))
+        (let ((method (match-string-no-properties 1))
+              (uri (match-string-no-properties 2)))
           `(,method . ,(es--fix-url uri)))
       (message "Could not find <method> <url> parameters!")
       nil)))
@@ -471,7 +510,7 @@ in which case it prompts the user."
   (cl-case command
     (prefix (let ((sym (company-grab-symbol)))
               (if (string-match "\"\\(.*\\)\"?" sym)
-                  (match-string 1 sym)
+                  (match-string-no-properties 1 sym)
                 sym)))
     (candidates
      (all-completions
@@ -488,7 +527,7 @@ in which case it prompts the user."
     (prefix (and (derived-mode-p 'es-mode)
                  (let ((sym (company-grab-symbol)))
                    (if (string-match "\"\\(.*\\)\"?" sym)
-                       (match-string 1 sym)
+                       (match-string-no-properties 1 sym)
                      sym))))
     (candidates
      (cl-remove-if-not
@@ -497,40 +536,50 @@ in which case it prompts the user."
     (annotation (es-extract-type arg))
     (meta (es-extract-summary arg))))
 
-(defun es-result--handle-response (status &optional results-buffer-name)
-  "Handles the response from the server returns after sending a query."
-  (let ((http-results-buffer (current-buffer))
-        (http-status-code url-http-response-status)
-        (http-content-type url-http-content-type)
-        (http-content-length url-http-content-length))
-    (set-buffer
-     (get-buffer-create results-buffer-name))
+(defun es-extract-warnings ()
+  "Extract the warning headers from the current buffer. Assumes
+  the buffer contains the result of a `url-retrieve' request."
+  (goto-char (point-min))
+  (when (re-search-forward "Warning: " (point-max) t)
+    (let ((start (point))
+          (end (progn (end-of-line) (point))))
+      (buffer-substring-no-properties start end))))
+
+(defun es-result--handle-response (data response error-thrown)
+  "Handles the response from the server after sending a request."
+  (let ((buffer-read-only nil)
+        (http-warnings (request-response-header response "warning"))
+        (http-content-type (request-response-header response "content-type"))
+        (http-content-length (request-response-header response "content-length"))
+        (http-status-code (request-response-status-code response)))
     (message "Response: Status: %S Content-Type: %S (%s bytes)"
              http-status-code http-content-type http-content-length)
-    (let ((buffer-read-only nil))
-      (delete-region (point-min) (point-max))
-      (if (or (equal 'connection-failed (cl-cadadr status))
-              (not (numberp http-status-code)))
-          (progn
-            (insert "ERROR: Could not connect to server.")
-            (setq mode-name (format "ES[failed]")))
-        (es-result-mode)
-        (url-insert http-results-buffer)
-        (cond
-         ((and (>= http-status-code 200) (<= http-status-code 299))
-          (run-hook-with-args 'es-response-success-functions
-                              http-status-code
-                              http-content-type
-                              (current-buffer)))
-         (t
-          (run-hook-with-args 'es-response-failure-functions
-                              http-status-code
-                              http-content-type
-                              (current-buffer))))
-        (setq mode-name "ES[finished]")))))
+    (erase-buffer)
+    (if error-thrown
+        (if data
+            (insert data)
+          (insert "ERROR: Could not connect to server."))
+      (when http-warnings
+        (insert "// Warning: " http-warnings "\n"))
+      (insert data))
+    (es-result-mode)
+    (cond
+     ((and (>= http-status-code 200) (<= http-status-code 299))
+      (run-hook-with-args 'es-response-success-functions
+                          http-status-code
+                          http-content-type
+                          (current-buffer)))
+     (t
+      (run-hook-with-args 'es-response-failure-functions
+                          http-status-code
+                          http-content-type
+                          (current-buffer))))
+    (if error-thrown
+        (setq mode-name "ES[failed]")
+      (setq mode-name "ES[finished]"))))
 
-(defun es--warn-on-delete-yes-or-no-p ()
-  (or (not (string= "DELETE" (upcase url-request-method)))
+(defun es--warn-on-delete-yes-or-no-p (request-method)
+  (or (not (string= "DELETE" (upcase request-method)))
       (not es-warn-on-delete-query)
       (yes-or-no-p
        ;; This will not font-lock if `yes-or-no-p' is aliased to
@@ -555,36 +604,68 @@ in which case it prompts the user."
 
 (defvar es--query-number 0)
 
-(defun es--execute-region ()
+(defun es--maybe-show-results-buffer (es-results-buffer)
+  "If the es-results-buffer exists, show it in the other window
+without changing focus. Does nothing if the buffer is already
+visible or doesn't exist."
+  (save-selected-window
+    ;; We want 2 buffers next to each other if it's not already visible, so
+    ;; delete other buffers
+    (when (and es-results-buffer
+               (not (get-buffer-window es-results-buffer)))
+      (view-buffer-other-window es-results-buffer))))
+
+(defun es--serialize-request-data(_request-data)
+  "Serialize tripple-quote json to utf-8 string format"
+  (let* ((_mapcar_index  0))
+    (string-join
+     (mapcar
+      (lambda(arg)
+        (setq _mapcar_index (+ 1 _mapcar_index))
+        (if (= (mod _mapcar_index 2) 0)
+            (replace-regexp-in-string (regexp-quote "\"") "\\\\\""
+                                      (string-join (split-string arg "\n") ""))
+          arg)
+        )
+      (split-string (concat _request-data "")  "\"\"\""))
+     "\"")))
+
+(defun es--execute-string (request-data-input)
   "Submits the active region as a query to the specified
 endpoint. If the region is not active, the whole buffer is
 used. Uses the params if it can find them or alternativly the
 vars."
-  (let* ((beg (if (region-active-p) (region-beginning) (point-min)))
-         (end (if (region-active-p) (region-end) (point-max)))
-         (url-request-extra-headers
-          '(("Content-Type" . "application/json; charset=UTF-8")))
+  (let* ((request-data (es--serialize-request-data request-data-input))
          (params (or (es--find-params)
                      `(,(es-get-request-method) . ,(es-get-url))))
          (url (es--munge-url (cdr params)))
-         (url-request-method (car params))
-         (url-request-data (encode-coding-string
-                            (buffer-substring-no-properties beg end) 'utf-8))
-         (result-buffer-name (if (zerop es--query-number)
-                                 (format "*ES: %s*" (buffer-name))
-                               (format "*ES: %s [%d]*"
-                                       (buffer-name)
-                                       es--query-number))))
-    (when (es--warn-on-delete-yes-or-no-p)
-      (message "Issuing %s against %s" url-request-method url)
-      (url-retrieve url 'es-result--handle-response (list result-buffer-name))
-      (setq es-results-buffer (get-buffer-create result-buffer-name))
-      (save-selected-window
-        ;; We want 2 buffers next to each other if it's not already visible, so
-        ;; delete other buffers
-        (when (not (get-buffer-window es-results-buffer))
-          (delete-other-windows)
-          (view-buffer-other-window es-results-buffer))))))
+         (url-request-method (car params)))
+    (let ((result-buffer-name (if (zerop es--query-number)
+                                  (format "*ES: %s*" (buffer-name))
+                                (format "*ES: %s [%d]*"
+                                        (buffer-name)
+                                        es--query-number))))
+      (when (es--warn-on-delete-yes-or-no-p url-request-method)
+        (let ((results-buffer (get-buffer-create result-buffer-name)))
+          (message "Issuing %s against %s" url-request-method url)
+          (request
+           url
+           :type url-request-method
+           :parser 'buffer-string
+           :headers es-default-headers
+           :data (let* ((utf-raw (encode-coding-string request-data 'utf-8))
+                        (utf-trimmed (string-trim utf-raw)))
+                   (if (string= "" utf-trimmed)
+                       utf-trimmed
+                     utf-raw))
+           :timeout 600 ;; timeout of 10 minutes
+           :complete (cl-function
+                      (lambda (&key data response error-thrown &allow-other-keys)
+                        (let ((utf-data (decode-coding-string data 'utf-8)))
+                          (with-current-buffer results-buffer
+                            (es-result--handle-response utf-data response error-thrown))))))
+          (setq es-results-buffer results-buffer)
+          (es--maybe-show-results-buffer es-results-buffer))))))
 
 (defun es--at-current-header-p ()
   "Returns t if at on a header line, nil otherwise."
@@ -594,13 +675,13 @@ vars."
   "Sets point to the beginning of the request body and mark at
 the end. May modify the current point."
   (interactive)
-  (let ((p (point))
-        startpoint
+  (let (startpoint
         endpoint)
     (beginning-of-line)
     (forward-line 1)
-    (es-goto-previous-request)
-    (forward-line 1)
+    (if (es-goto-previous-request)
+        (forward-line 1)
+      (goto-char (point-min)))
     (setq startpoint (point))
     (if (es-goto-next-request)
         (forward-line -1)
@@ -609,6 +690,23 @@ the end. May modify the current point."
     (goto-char startpoint)
     (set-mark endpoint)))
 
+(defun es-get-request-body ()
+  "Sets point to the beginning of the request body and mark at
+the end. May modify the current point."
+  (interactive)
+  (let (startpoint
+        endpoint)
+    (beginning-of-line)
+    (forward-line 1)
+    (if (es-goto-previous-request)
+        (forward-line 1)
+      (goto-char (point-min)))
+    (setq startpoint (point))
+    (if (es-goto-next-request)
+        (forward-line -1)
+      (goto-char (point-max)))
+    (buffer-substring-no-properties startpoint (point))))
+
 (defun es-goto-previous-request ()
   "Advance the point to the previous parameter declaration, if
 available. Returns truthy if one was found, nil otherwise."
@@ -644,13 +742,11 @@ the buffer is executed from top to bottom."
    (when prefix
      (goto-char (point-min))
      (setq es--query-number 1))
-   (es-mark-request-body)
-   (es--execute-region)
+   (es--execute-string (es-get-request-body))
    (when prefix
      (while (es-goto-next-request)
        (setq es--query-number (1+ es--query-number))
-       (es-mark-request-body)
-       (es--execute-region))
+       (es--execute-string (es-get-request-body)))
      (setq es--query-number 0))))
 
 (defun es-result-show-response ()
@@ -676,7 +772,8 @@ the buffer is executed from top to bottom."
   (make-local-variable 'es-result-response)
   ;; Also set comment chars in results mode
   (setq-local comment-start "// ")
-  (setq-local comment-start-skip "//+[\t ]*"))
+  (setq-local comment-start-skip "//+[\t ]*")
+  (run-mode-hooks 'es-result-mode-hook))
 
 (defun es-indent-line ()
   "Indent current line as ES code. Uses the same indention as js-mode."
@@ -712,12 +809,12 @@ the buffer is executed from top to bottom."
       ;; builtins for REST
       (,(concat "^\\s-*\\("
                 (regexp-opt es-http-builtins)
-                "\\)\\s-+\\(http://[^[:space:]\n]+\\)")
+                "\\)[[:space:]]+\\(http://[^[:space:]\n]+\\)")
        (1 font-lock-builtin-face t)
        (2 font-lock-variable-name-face t))
       (,(concat "^\\("
                 (regexp-opt es-http-builtins-all)
-                "\\) \\([^[:space:]\n]*\\)")
+                "\\)[[:space:]]+\\([^[:space:]\n]*\\)")
        (1 font-lock-builtin-face t)
        (2 font-lock-variable-name-face t))
       ;; keywords for fields usually specified
@@ -744,6 +841,7 @@ the buffer is executed from top to bottom."
     ;; / is a punctuation character and is the first and second
     ;; character of a two letter comment starter.
     (modify-syntax-entry ?/ ". 12" st)
+    (modify-syntax-entry ?# "<" st)
     ;; newline is the end of a comment.
     (modify-syntax-entry ?\n ">" st)
     st)
@@ -756,6 +854,7 @@ the buffer is executed from top to bottom."
     (define-key map (kbd "C-c C-c") 'es-execute-request-dwim)
     (define-key map (kbd "C-c C-u") 'es-set-endpoint-url)
     (define-key map (kbd "C-c C-m") 'es-set-request-method)
+    (define-key map (kbd "C-c M-w") 'es-copy-as)
     map)
   "Keymap for `es-mode'.")
 
@@ -791,6 +890,23 @@ the buffer is executed from top to bottom."
 ;;;###autoload
 (add-to-list 'auto-mode-alist '("\\.es\\'" . es-mode))
 
+(setq es-mode-snippets-dir
+      (when (and (boundp 'load-file-name) load-file-name)
+        (file-name-directory load-file-name)))
+
+;;;###autoload
+(defun es-mode-snippets-initialize ()
+  (when es-mode-snippets-dir
+    (let ((snip-dir (expand-file-name "snippets" es-mode-snippets-dir)))
+      (when (and (boundp 'yas-snippet-dirs)
+                 (file-exists-p snip-dir))
+        (add-to-list 'yas-snippet-dirs snip-dir t))
+      (yas-load-directory snip-dir))))
+
+;;;###autoload
+(eval-after-load 'yasnippet
+  '(es-mode-snippets-initialize))
+
 (provide 'es-mode)
 
 ;;; es-mode.el ends here
